#import "Controller.h"
#import <MulleCipher/MulleCipher.h>
#import "NSMutableData+Parse.h"


@implementation Controller


- (void) encrypt:(id) sender
{
   NSData                   *plaintext;
   id                       code;
   NSData                   *key;
   MulleSymmetricCipher     *cipher;
   MulleSymmetricCipherKey  *cryptKey;

NS_DURING   
// UI stuff
   cipher = [[MulleSymmetricCipher registeredCiphers] objectAtIndex:[cryptPopUpButton indexOfSelectedItem]];

   plaintext = [[originalView string] dataUsingEncoding:NSISOLatin1StringEncoding];
   key       = [[keyView stringValue] dataUsingEncoding:NSISOLatin1StringEncoding];

   if( ! plaintext)
   {
      NSBeep();
      NS_VOIDRETURN;
   }
   
// encryption
   cryptKey = [[cipher keyClass] keyWithData:key
                                forDirection:MulleCipherForEncryption];
   [keyHexView setString:[[cryptKey data] description]];
   
   if( [inplaceButton state] == NSOnState)
   {
      code = [[plaintext mutableCopy] autorelease];
      if( [encodeButton state] == NSOnState)
         [cipher encryptDataWithFooter:code
                               withKey:cryptKey];
      else
         [cipher encryptData:code
                     withKey:cryptKey];
   }
   else
      if( [encodeButton state] == NSOnState)
         code = [cipher encryptedDataWithFooter:plaintext
                                        withKey:cryptKey];
      else
         code = [cipher encryptedData:plaintext
                              withKey:cryptKey];

// more UI stuff

   [cryptedView setString:[code description]];
   [decryptedView setString:@""];
NS_HANDLER
   NSRunAlertPanelRelativeToWindow([localException name], [localException reason], @"OK", nil, nil,  window, nil);
NS_ENDHANDLER   
}



- (void) decrypt:(id) sender
{
   NSData                   *code;
   id                       plaintext;
   NSData                   *key;
   NSString                 *s;
   MulleSymmetricCipher     *cipher;
   MulleSymmetricCipherKey  *cryptKey;

NS_DURING   
// UI stuff
   cipher = [[MulleSymmetricCipher registeredCiphers] objectAtIndex:[cryptPopUpButton indexOfSelectedItem]];

   code = [NSMutableData dataWithDescription:[cryptedView string]];
   key  = [[keyView stringValue] dataUsingEncoding:NSISOLatin1StringEncoding];

   if( ! code)
   {
      NSBeep();
      NS_VOIDRETURN;
   }
   
   // decryption
   cryptKey = [[cipher keyClass] keyWithData:key
                                forDirection:MulleCipherForDecryption];
   [keyHexView setString:[[cryptKey data] description]];
   
   if( [inplaceButton state] == NSOnState)
   {
      plaintext = [[code mutableCopy] autorelease];
      if( [encodeButton state] == NSOnState)
         [cipher decryptDataWithFooter:plaintext
                                 withKey:cryptKey];
      else
         [cipher decryptData:plaintext
                     withKey:cryptKey];
   }
   else
   {
      if( [encodeButton state] == NSOnState)
         plaintext = [cipher decryptedDataWithFooter:code
                                             withKey:cryptKey];
      else
         plaintext = [cipher decryptedData:code
                                   withKey:cryptKey];
   }
   

   s = [[[NSString alloc] initWithData:plaintext
                             encoding:NSISOLatin1StringEncoding] autorelease];
   [decryptedView setString:s];
NS_HANDLER
   NSRunAlertPanelRelativeToWindow([localException name], [localException reason], @"OK", nil, nil,  window, nil);
NS_ENDHANDLER   
}


- (void) cycle:(id) sender
{
   [self encrypt:self];
   [self decrypt:self];
}



- (void) ultra:(id) sender
{
   NSDictionary             *info;
   NSData                   *plaintext;
   id                       code;
   NSData                   *key;
   MulleSymmetricCipher     *cipher;
   NSString                 *s;

NS_DURING
// UI stuff
   cipher = [[MulleSymmetricCipher registeredCiphers] objectAtIndex:[cryptPopUpButton indexOfSelectedItem]];

   plaintext = [[originalView string] dataUsingEncoding:NSISOLatin1StringEncoding];
   key       = [[keyView stringValue] dataUsingEncoding:NSISOLatin1StringEncoding];

   if( ! plaintext)
   {
      NSBeep();
      NS_VOIDRETURN;
   }

// encryption
   info = [cipher crypt:plaintext
               password:key];

// more UI stuff

   code = [info objectForKey:MulleCipherCiphertext];
   [cryptedView setString:[code description]];


   if( ! code)
   {
      NSBeep();
      NS_VOIDRETURN;
   }

   plaintext = [cipher decrypt:info
                      password:key];

   s = [[[NSString alloc] initWithData:plaintext
                             encoding:NSISOLatin1StringEncoding] autorelease];
   [decryptedView setString:s];
   [keyHexView setString:@"unavailable"];
NS_HANDLER
   NSRunAlertPanelRelativeToWindow([localException name], [localException reason], @"OK", nil, nil,  window, nil);
NS_ENDHANDLER
}


- (void) digestify:(id) sender
{
   NSData   *data;
   NSData   *digest;

NS_DURING   
   data = [[originalView string] dataUsingEncoding:NSISOLatin1StringEncoding];
   
   if( [[digestPopUpButton selectedItem] tag] == 0)
      digest = [data md5Digest];
   else
      digest = [data sha1Digest];

   [cryptedView setString:[digest description]];
   [decryptedView setString:@"No decryption for digests"];
   [keyHexView setString:@"unavailable"];
NS_HANDLER
   NSRunAlertPanelRelativeToWindow([localException name], [localException reason], @"OK", nil, nil,  window, nil);
NS_ENDHANDLER   
}


- (void) awakeFromNib
{
   [cryptPopUpButton removeAllItems];
   [cryptPopUpButton addItemsWithTitles:[MulleSymmetricCipher registeredCipherNames]];
}



@end
